<?php

/**
 * Sends requests to the server returns error if a timeout occurs.
 */
class ImlApi {

  const maxFileReadingStringLength = 4096;

  /**
   * @var string
   *  Well-formed server url to connect with
   */
  private $serverUrl;

  /**
   * @var int
   *  Timeout for server communications in seconds.
   */
  private $timeout;

  /**
   * @var string
   *  Api login.
   */
  private $login;

  /**
   * @var string
   *  Api password.
   */
  private $password;

  /**
   * @var boolean
   *  If working in test mode.
   */
  private $isTestMode;

  /**
   * @param string $server_url
   *  Server url.
   * @param int $timeout
   *  Timeout in seconds.
   * @param string $login
   *  Api login.
   * @param string $password
   *  Api password.
   * @param boolean $is_test_mode
   *  Is test mode.
   */
  public function __construct($server_url, $timeout, $login, $password, $is_test_mode) {
    $this->serverUrl = $server_url;
    $this->timeout = $timeout;
    $this->login = $login;
    $this->password = $password;
    $this->isTestMode = $is_test_mode;
  }

  /**
   * @param array $parameters
   *
   * @return array
   */
  public function getOrderStatuses(array $parameters) {
    return $this->sendPostRequest('GetStatuses', $parameters);
  }

  /**
   * @param array $parameters
   *
   * @return array
   */
  public function saveOrder(array $parameters) {
    return $this->sendPostRequest('CreateOrder', $parameters);
  }

  /**
   * @param string $barCode
   *
   * @return string
   */
  public function printBarcode($barCode) {
    return $this->sendGetRequest("PrintBar?Barcode={$barCode}");
  }

  /**
   * @param $urlSuffix
   *
   * @return mixed
   */
  private function sendGetRequest($urlSuffix) {
    $headers = array(
      'Authorization' => "Basic " . base64_encode("{$this->login}:{$this->password}"),
      'Accept' => '*/*',
    );

    $response = drupal_http_request("{$this->serverUrl}{$urlSuffix}", array(
      'headers' => $headers,
      'method' => 'GET',
    ));

    if (!(isset($response->status_message) && $response->status_message == 'OK')) {
      return array('Errors' => array($response->error));
    }
    if (isset($response->headers['content-type']) && $response->headers['content-type'] == 'application/json; charset=utf-8') {
      return json_decode($response->data, TRUE);
    }

    return $response->data;
  }

  /**
   * @param string $urlSuffix
   * @param array $parameters
   *
   * @return array
   *  Resulted data structure representing decoded xml or containing error information.
   */
  private function sendPostRequest($urlSuffix, array $parameters) {
    if ($this->isTestMode) {
      $parameters['Test'] = 'True';
    }
    
    try {
      $context = $this->createStreamContext($parameters);

      $communicationStartTime = time();
      $fp = fopen("{$this->serverUrl}{$urlSuffix}", 'rb', NULL, $context);
      if (!$fp) {
        $this->throwConnectionException($communicationStartTime, t("Problem occurred while connecting to API's server."));
      }
      stream_set_blocking($fp, FALSE);

      $response = $this->readResponse($fp);
      fclose($fp);

      if (empty($response)) {
        $this->throwConnectionException($communicationStartTime, t("Problem occurred while reading data from API's server."));
      }

      return json_decode($response, TRUE);
    }
    catch (Exception $exception) {
      return array('Errors' => array($exception->getMessage()));
    }
  }

  /**
   * @param int $communicationStartTime
   *
   * @return bool
   */
  private function isTimedOut($communicationStartTime) {
    return (time() - $communicationStartTime) >= $this->timeout;
  }

  /**
   * @param int $communicationStartTime
   * @param string $message
   * 
   * @throws Exception
   */
  private function throwConnectionException($communicationStartTime, $message) {
    $timeoutMsg = $this->isTimedOut($communicationStartTime) ? t('API server response time exceeded (@timeout)', array('@timeout' => $this->timeout)) : '';
    throw new Exception($message . ' ' . $timeoutMsg);
  }

  /**
   * @param mixed $fp
   *  File pointer.
   *
   * @return string
   *  Read string.
   */
  private function readResponse($fp) {
    $response = '';

    while (!feof($fp)) {
      $response .= fgets($fp, self::maxFileReadingStringLength);
    }

    return $response;
  }

  /**
   * @param $parameters
   *
   * @return resource
   */
  private function createStreamContext(array $parameters) {
    $auth = base64_encode("{$this->login}:{$this->password}");
    $params = array(
      'http' => array(
        'method' => 'POST',
        'content' => http_build_query($parameters),
        'timeout' => $this->timeout,
        'header' => "Content-type: application/x-www-form-urlencoded;charset=\"utf-8\"\r\nAuthorization: Basic {$auth}",
      ),
      'ssl' => array(
        'verify_peer' => FALSE,
        'verify_host' => 1,
      ),
    );

    return stream_context_create($params);
  }

}
